//
//  BusinessController.swift
//  tfc
//
//  Created by jorge on 05/05/15.
//  Copyright (c) 2015 Jorge Tercero.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.
//

import UIKit
import CoreData

class BusinessController: UIViewController, BusinessDetailsDelegate, BusinessReviewsDelegate, UITableViewDelegate, UITableViewDataSource {
    
    var biz: Business!
    var bizInDB: NSManagedObject?
    var reviews: [Review]?
    
    @IBOutlet weak var mainPhotoView: UIImageView!
    @IBOutlet weak var bizNameLabel: UILabel!
    @IBOutlet weak var bizAddressLabel: UILabel!
    @IBOutlet weak var bizLocalityLabel: UILabel!
    @IBOutlet weak var bizTelephoneButton: UIButton!
    @IBOutlet weak var mapButton: UIButton!
    @IBOutlet weak var favoritesButton: UIButton!
    @IBOutlet weak var reviewsTable: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.bizNameLabel.text = biz.name

        reviewsTable.hidden = true
        
        let appDelegate = UIApplication.sharedApplication().delegate as! AppDelegate
        let localService = appDelegate.localService!
        localService.bizReviewsDelegate = self
        localService.reviews(biz.slug)
        
        if let url = biz.bigPhotoURL {
            let data = NSData(contentsOfURL: NSURL(string:url)!)
            mainPhotoView.image = UIImage(data: data!)
        } else {
            mainPhotoView.image = UIImage(named: "grey_square")
        }
        
        if let address = biz.address {
            self.bizAddressLabel.text = address
        }
        if let city = biz.locality {
            self.bizLocalityLabel.text = city
        }
        if let phone = biz.telephone {
            self.bizTelephoneButton.setTitle(phone, forState: .Normal)
        } else {
            self.bizTelephoneButton.hidden = true;
        }
        
        if !biz.hasCoordinates() {
            self.mapButton.hidden = true
        }
        
        if self.isBusinessInFavorites() {
            self.favoritesButton.setTitle("quitar de favoritos", forState: .Normal)
        }
    }

    func businessDetailsLoaded(biz: Business) {
        println(">>>>> businessDetailsLoaded")
        self.biz = biz
    }
    
    // MARK: user actions
    
    @IBAction func makePhoneCall(sender: AnyObject) {
        if let url = NSURL(string: "tel://\(biz.telephone)") {
            UIApplication.sharedApplication().openURL(url)
        }
    }
    
    // MARK: favorites management
    
    @IBAction func addOrRemoveFavorite(sender: AnyObject) {
        let appDelegate = UIApplication.sharedApplication().delegate as! AppDelegate
        let context = appDelegate.managedObjectContext!
        
        if self.isBusinessInFavorites() {
            context.deleteObject(bizInDB!)
            self.favoritesButton.setTitle("añadir a favoritos", forState: .Normal)
        } else {
            let dbbiz = NSEntityDescription.insertNewObjectForEntityForName("Business", inManagedObjectContext: context) as! NSManagedObject
        
            dbbiz.setValue(biz.name, forKey: "name")
            dbbiz.setValue(biz.slug, forKey: "slug")
            dbbiz.setValue(biz.address, forKey: "address")
            dbbiz.setValue(biz.locality, forKey: "locality")
            dbbiz.setValue(biz.photoURL, forKey: "photoURL")
            dbbiz.setValue(biz.summary, forKey: "summary")
            dbbiz.setValue(biz.telephone, forKey: "telephone")
            dbbiz.setValue(biz.latitude, forKey: "latitude")
            dbbiz.setValue(biz.longitude, forKey: "longitude")
            
            self.favoritesButton.setTitle("quitar de favoritos", forState: .Normal)
        }
        
        var error: NSError?
        if !context.save(&error) {
            println("Could not save \(error), \(error?.userInfo)")
        }
    }
    
    func isBusinessInFavorites() -> Bool {
        let appDelegate = UIApplication.sharedApplication().delegate as! AppDelegate
        let managedContext = appDelegate.managedObjectContext!
        
        let fetchRequest = NSFetchRequest(entityName:"Business")
        fetchRequest.predicate = NSPredicate(format: "slug = %@", biz.slug)
        
        var error: NSError?
        let fetchedResults = managedContext.executeFetchRequest(fetchRequest, error: &error) as? [NSManagedObject]
        
        if let results = fetchedResults {
            if results.count>0 {
                bizInDB = results[0]
                return true
            } else {
                return false
            }
        } else {
            println("Could not fetch \(error), \(error!.userInfo)")
            return false
        }
    }

    
    // MARK: navigation
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject!) {
        if segue.identifier == "showMap" {
            let mapController = segue.destinationViewController as! BusinessMapController
            mapController.biz = biz
        } else if segue.identifier == "showReview" {
            let index = reviewsTable.indexPathForSelectedRow()
            let review = reviews?[index!.row]
            let reviewController = segue.destinationViewController as! ReviewController
            reviewController.review = review
        }
    }
    
    // MARK: BusinessReviewsDelegate
    
    func businessReviewsLoaded(reviews: [Review]) {
        self.reviews = reviews
        dispatch_sync(dispatch_get_main_queue()) {
            if reviews.count > 0 {
                self.reviewsTable.hidden = false
                self.reviewsTable.reloadData()
            }
        }
    }

    // MARK: UITableViewDataSource reviews table

    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if let revs = reviews {
            return revs.count
        } else {
            return 0
        }
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell
    {
        var cell = UITableViewCell(style: UITableViewCellStyle.Subtitle, reuseIdentifier: nil)
        if let review = reviews?[indexPath.row] {
            cell.textLabel!.text = review.title
            cell.detailTextLabel!.text = review.content
            //cell.imageView?.image = UIImage(named: "grey_square")
        }
        return cell
    }
    
    // MARK: UITableViewDelegate
    
    func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        self.performSegueWithIdentifier("showReview", sender: self)
    }
}
