//
//  LocalService.swift
//  tfc
//
//  Created by jorge on 11/04/15.
//  Copyright (c) 2015 Jorge Tercero.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
//  SOFTWARE.
//

import Foundation
import CoreLocation

class LocalService: NSObject, CLLocationManagerDelegate {
    
    let BASE_URL = "https://api.11870.com/api/v2/"
    let APP_TOKEN = "XXXX" // puedes conseguir un api token en https://sites.google.com/site/api11870/
    let SEARCH = "search"
    let SITE_DETAILS = "site-details"
    let SITE_REVIEWS = "site-reviews"
    let FEATURED_BIZ = "featured-site"
    
    let parser = OOSParser()
    
    let locationManager = CLLocationManager()
    var location: CLLocation?
    
    var searchDelegate: SearchBusinessesDelegate?
    var bizDetailsDelegate: BusinessDetailsDelegate?
    var bizReviewsDelegate: BusinessReviewsDelegate?
    var featuredBizDelegate: FeaturedBusinessDelegate?
    
    override init() {
        super.init()
        locationManager.delegate = self
        if CLLocationManager.authorizationStatus() == .NotDetermined {
            locationManager.requestWhenInUseAuthorization()
        }
    }
    
    // MARK: location manager
    
    //
    // el sistema nos notifica si el usuario autoriza compartir su ubicación
    //
    func locationManager(manager: CLLocationManager!, didChangeAuthorizationStatus status: CLAuthorizationStatus)
    {
        if status == .AuthorizedWhenInUse {
            manager.startUpdatingLocation()
        }
    }

    //
    // el sistema nos notifica la localización del usuario
    //
    func locationManager(manager: CLLocationManager!, didUpdateLocations locations: [AnyObject]!) {
        let firstLocation = self.location == nil
        self.location = locations.last as? CLLocation
        if firstLocation {
            if let coord = self.location?.coordinate {
                NSNotificationCenter.defaultCenter().postNotificationName("TFCLocationReady", object: self)
            }
        }
    }
    
    // MARK: 11870 API
    
    //
    // búsqueda texto libre
    //
    func search(query: NSString)
    {
        let baseURL = BASE_URL + SEARCH
        let url = NSURLComponents(string: baseURL)!
        url.query = "appToken=\(APP_TOKEN)&q=\(query)&alt=json&count=30"
        
        // creamos la task que se llamará en el background thread
        let task = NSURLSession.sharedSession().dataTaskWithURL(url.URL!) {(data, response, error) in
            let results = self.parser.parseSearchResults(data)
            if let delegate = self.searchDelegate {
                delegate.searchResults(results)
            }
        }
        
        // lanzamos la task en background
        task.resume()
    }
    
    //
    // negocios alrededor mío
    //
    func aroundMe() {
        
        let baseURL = BASE_URL + SEARCH
        let url = NSURLComponents(string: baseURL)!
        if let coord = self.location?.coordinate {
            url.query = "appToken=\(APP_TOKEN)&alt=json&count=30&lat=\(coord.latitude)&lon=\(coord.longitude)&radius=0.5"
        } else {
            return
        }
        
        let task = NSURLSession.sharedSession().dataTaskWithURL(url.URL!) {(data, response, error) in
            let results = self.parser.parseSearchResults(data)
            if let delegate = self.searchDelegate {
                delegate.searchResults(results)
            }
        }
        
        task.resume()
    }
    
    func business(slug: NSString) {
        
        let baseURL = "\(BASE_URL)\(SITE_DETAILS)/\(slug)";
        let url = NSURLComponents(string: baseURL)!
        url.query = "appToken=\(APP_TOKEN)&alt=json"
        
        let task = NSURLSession.sharedSession().dataTaskWithURL(url.URL!) {(data, response, error) in
            var parseError: NSError?
            let parsedObject: NSDictionary = NSJSONSerialization.JSONObjectWithData(
                data,
                options: NSJSONReadingOptions.AllowFragments,
                error:&parseError) as! NSDictionary
        }
        
        task.resume()
    }

    func reviews(slug: NSString) {
        
        let baseURL = "\(BASE_URL)\(SITE_REVIEWS)/\(slug)";
        let url = NSURLComponents(string: baseURL)!
        url.query = "appToken=\(APP_TOKEN)&alt=json&withContent=true&count=50"
        
        let task = NSURLSession.sharedSession().dataTaskWithURL(url.URL!) {(data, response, error) in
            let reviews = self.parser.parseBusinessReviews(data)
            if let delegate = self.bizReviewsDelegate {
                delegate.businessReviewsLoaded(reviews)
            }
        }
        
        task.resume()
    }
    
    func featuredBusiness() {
        
        let baseURL = BASE_URL + FEATURED_BIZ
        let url = NSURLComponents(string: baseURL)!
        if let coord = self.location?.coordinate {
            url.query = "appToken=\(APP_TOKEN)&alt=json&lat=\(coord.latitude)&lon=\(coord.longitude)&kId=1&count=2"
        } else {
            url.query = "appToken=\(APP_TOKEN)&alt=json&lat=40.4&lon=-3.68&kId=1&count=2"
        }
        
        let task = NSURLSession.sharedSession().dataTaskWithURL(url.URL!) {(data, response, error) in
            let results = self.parser.parseFeaturedBusiness(data)
            if let delegate = self.featuredBizDelegate {
                delegate.featuredBusinessLoaded(results)
            }
        }
        
        task.resume()
    }
}

// MARK: delegates

protocol SearchBusinessesDelegate {
    func searchResults([Business])
}

protocol BusinessDetailsDelegate {
    func businessDetailsLoaded(Business)
}

protocol BusinessReviewsDelegate {
    func businessReviewsLoaded([Review])
}

protocol FeaturedBusinessDelegate {
    func featuredBusinessLoaded([Business])
}

